/* eslint-disable no-param-reassign */
import { RuntimeError } from '@ibiz-template/core';
import {
  getGridUploadProps,
  useGridCellEditor,
  useNamespace,
} from '@ibiz-template/vue-util';
import { CreateElement, defineComponent, ref, Ref, watch } from 'vue';
import { UploadFile } from '../ibiz-upload/ibiz-upload';
import { IBizGridFileUploadPopover } from './ibiz-grid-file-upload-popover';

export const IBizGridFileUpload = defineComponent({
  name: 'IBizGridFileUpload',
  props: getGridUploadProps(),
  setup(props) {
    const ns = useNamespace('grid-upload');

    // 上传文件路径
    const uploadUrl: Ref<string> = ref('');

    // 下载文件路径
    const downloadUrl: Ref<string> = ref('');

    // 文件列表
    const fileList: Ref<UploadFile[]> = ref([]);

    // data响应式变更基础路径
    watch(
      () => props.data,
      newVal => {
        if (newVal) {
          const urls = props.controller.calcBaseUrl(newVal);
          uploadUrl.value = urls.uploadUrl;
          downloadUrl.value = urls.downloadUrl;
        }
      },
      { immediate: true, deep: true },
    );

    // 值响应式变更
    watch(
      () => props.value,
      newVal => {
        fileList.value = !newVal ? [] : JSON.parse(newVal);
      },
      { immediate: true },
    );

    watch(
      fileList,
      newVal => {
        // 变更后且下载基础路径存在时解析
        if (newVal?.length && downloadUrl.value) {
          newVal.forEach((file: IData) => {
            file.url =
              file.url || downloadUrl.value.replace('%fileId%', file.id);
          });
        }
      },
      { immediate: true },
    );

    watch(
      downloadUrl,
      newVal => {
        // 变更后且下载基础路径存在时解析
        if (newVal && fileList.value.length) {
          fileList.value.forEach((file: IData) => {
            file.url = downloadUrl.value.replace('%fileId%', file.id);
          });
        }
      },
      { immediate: true },
    );

    const { isInfoMode, componentRef, onOperateChange, onChange } =
      useGridCellEditor(props.rowDataChange, props.rowSave, {
        isDelaySave: false,
      });

    const emitValue = (files: UploadFile[]) => {
      const value: string | null =
        files.length > 0
          ? JSON.stringify(
              files.map(file => ({ name: file.name, id: file.id })),
            )
          : null;
      console.log(value);

      onChange(value);
    };

    const openPopover = async () => {
      if (!componentRef.value) {
        throw new RuntimeError('容器元素不存在');
      }
      const el = componentRef.value.$el as HTMLElement;
      const elHeight = el.offsetHeight;
      const popover = ibiz.overlay.createPopover(
        (h: CreateElement) => {
          return h(IBizGridFileUploadPopover, {
            props: {
              value: fileList.value,
              uploadUrl: uploadUrl.value,
              downloadUrl: downloadUrl.value,
            },
            on: {
              close: (result: {
                isModified: boolean;
                resultFiles: UploadFile[];
              }) => {
                popover.dismiss(result);
              },
            },
          });
        },
        undefined,
        {
          noArrow: true,
          placement: 'bottom-start',
          offsetOpts: { mainAxis: -elHeight, crossAxis: 0 },
        },
      );
      await popover.present(el);
      const result = await popover.onWillDismiss<{
        isModified: boolean;
        resultFiles: UploadFile[];
      }>();
      if (result.isModified) {
        emitValue(result.resultFiles);
      }
    };

    watch(isInfoMode, async newVal => {
      if (!props.disabled && newVal === false) {
        onOperateChange(true);
        await openPopover();
        onOperateChange(false);
      }
    });

    return {
      ns,
      isInfoMode,
      fileList,
      componentRef,
      onOperateChange,
      onChange,
    };
  },
  render() {
    return (
      <grid-editor
        disabled={this.disabled}
        ref='componentRef'
        class={`${this.ns.b()}`}
      >
        {this.fileList.map(file => {
          return <span>{file.name}、</span>;
        })}
      </grid-editor>
    );
  },
});
